package main;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import com.github.stefanbirkner.systemlambda.SystemLambda;

import usermanagement.interaktiv.Messages;
import usermanagement.interaktiv.Zugriffsdialog;

public class SystemTest {
  private Zugriffsdialog dialog;

  @BeforeEach
  public void setUp() {
    this.dialog = new Zugriffsdialog();
  }

  private void anmelden(String login, String passwort)
      throws Exception {
    String[] inputs = { login, passwort };
    SystemLambda.withTextFromSystemIn(inputs)
        .execute(() -> {
          this.dialog.nutzerWechseln();
        });
  }

  private void sysadminHinzu(String login, String passwort)
      throws Exception {
    String[] inputs = { login, passwort, "1" };
    SystemLambda.withTextFromSystemIn(inputs)
        .execute(() -> {
          this.dialog.nutzerHinzufuegen();
        });
  }

  private void projektadminHinzu(String login, String passwort)
      throws Exception {
    String[] inputs = { login, passwort, "2" };
    SystemLambda.withTextFromSystemIn(inputs)
        .execute(() -> {
          this.dialog.nutzerHinzufuegen();
        });
  }

  private void entwicklerHinzu(String login, String passwort)
      throws Exception {
    String[] inputs = { login, passwort, "3" };
    SystemLambda.withTextFromSystemIn(inputs)
        .execute(() -> {
          this.dialog.nutzerHinzufuegen();
        });
  }

  private void passwortAendern(String alt, String neu) throws Exception {
    String[] inputs = { alt, neu };
    SystemLambda.withTextFromSystemIn(inputs)
    .execute(() -> {
      this.dialog.passwortAendern();
    });
  }

  private void loginAendern(String alt, String neu) throws Exception {
    String[] inputs = { alt, neu };
    SystemLambda.withTextFromSystemIn(inputs)
    .execute(() -> {
      this.dialog.loginVeraendern();
    });
  }

  @Test
  public void testStartOhneNutzer1() throws Exception {
    this.dialog = new Zugriffsdialog();
    
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.anmelden("x", "x");
    });
    Assertions.assertTrue(
        systemOut
            .contains(Messages.getString("Zugriffsdialog.27"))
            , "Nicht vorhandener Nutzer x kann sich anmelden");
  }

  @Test
  public void testStartOhneNutzer2() throws Exception {
    this.dialog = new Zugriffsdialog();
    
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.entwicklerHinzu("x", "x");
    });
    Assertions.assertTrue(
        systemOut
            .contains(Messages.getString("Zugriffsdialog.19"))
            , "Ohne erste Anmeldung kein neuer Entwickler");
  }

  @Test
  public void testStartOhneNutzer3() throws Exception {
    this.dialog = new Zugriffsdialog();
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.projektadminHinzu("x", "x");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.19"))
            , "Ohne erste Anmeldung kein neuer Projektadmin");
  }

  @Test
  public void testStartOhneNutzer4() throws Exception {
    this.dialog = new Zugriffsdialog();
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.sysadminHinzu("x", "x");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.19"))
            , "Ohne erste Anmeldung kein neuer Sysadmin");
  }

  @Test
  public void testStartOhneNutzer5() throws Exception {
    this.dialog = new Zugriffsdialog();
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.passwortAendern("admin", "x");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.23"))
            , "Ohne erste Anmeldung kein Passwort aendern");
  }

  @Test
  public void testStartOhneNutzer6() throws Exception {
    this.dialog = new Zugriffsdialog();
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.loginAendern("admin", "x");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.11"))
            , "Ohne erste Anmeldung kein Login aendern");
  }

  @Test
  public void testAdminNachAnforderungEingerichtet() throws Exception {
    this.dialog = new Zugriffsdialog();
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.anmelden("admin", "admin");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.26"))
            , "Nutzer (admin,admin) muss vorhanden sein");
  }

  @Test
  public void testAdminKannAdminEinrichten() throws Exception {
    this.dialog = new Zugriffsdialog();
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.anmelden("admin", "admin");
      this.sysadminHinzu("ad", "ad2");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.18"))
            , "Sysadmin muss neuen Sysadmin einrichten koennen");
  }

  @Test
  public void testNeuerAdminKannSichAnmelden() throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.sysadminHinzu("ad", "ad2");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.anmelden("ad", "ad2");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.26"))
            , "neuer Sysadmin muss sich anmelden koennen");
  }

  @Test
  public void testAdminKannProjektAdminEinrichten() throws Exception {
    this.dialog = new Zugriffsdialog();
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.anmelden("admin", "admin");
      this.projektadminHinzu("pro", "pro2");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.18"))
            , "Sysadmin muss neuen Projektadmin einrichten koennen");
  }

  @Test
  public void testNeuerProjektAdminKannSichAnmelden() throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.projektadminHinzu("pro", "pro2");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.anmelden("pro", "pro2");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.26"))
            , "neuer Projektadmin muss sich anmelden koennen");
  }

  @Test
  public void testAdminKannEntwicklerEinrichten() throws Exception {
    this.dialog = new Zugriffsdialog();
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.anmelden("admin", "admin");
      this.entwicklerHinzu("ent", "ent2");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.18"))
            , "Sysadmin muss neuen Entwickler einrichten koennen");
  }

  @Test
  public void testNeuerEntwicklerKannSichAnmelden() throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.entwicklerHinzu("ent", "ent2");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.anmelden("ent", "ent2");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.26"))
            , "neuer Entwickler muss sich anmelden koennen");
  }

  @Test
  public void testProjektAdminKannKeinenSysadminErzeugen() throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.projektadminHinzu("pro", "pro2");
    this.anmelden("pro", "pro2");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.sysadminHinzu("a", "b");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.19"))
            , "Projektadmin darf keinen Sysadmin erzeugen");
  }

  @Test
  public void testProjektAdminKannKeinProjektadminErzeugen() throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.projektadminHinzu("pro", "pro2");
    this.anmelden("pro", "pro2");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.projektadminHinzu("a", "b");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.19"))
            , "Projektadmin darf keinen Projektadmin erzeugen");
  }

  @Test
  public void testProjektAdminKannKeinenEntwicklerErzeugen() throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.projektadminHinzu("pro", "pro2");
    this.anmelden("pro", "pro2");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.entwicklerHinzu("a", "b");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.19"))
            , "Projektadmin darf keinen Entwickler erzeugen");
  }

  @Test
  public void testEntwicklerKannKeinenSysadminErzeugen() throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.entwicklerHinzu("ent", "ent2");
    this.anmelden("ent", "ent2");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.sysadminHinzu("a", "b");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.19"))
            , "Entwickler darf keinen Sysadmin erzeugen");
  }

  @Test
  public void testEntwicklerKannKeinProjektadminErzeugen() throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.entwicklerHinzu("ent", "ent2");
    this.anmelden("ent", "ent2");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.projektadminHinzu("a", "b");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.19"))
            , "Entwickler darf keinen Projektadmin erzeugen");
  }

  @Test
  public void testEntwicklerKannKeinenEntwicklerErzeugen() throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.entwicklerHinzu("ent", "ent2");
    this.anmelden("ent", "ent2");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.entwicklerHinzu("a", "b");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.19"))
            , "Entwickler darf keinen Entwickler erzeugen");
  }

  @Test
  public void testEntwicklerKannKorrektesPasswortAendern() throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.entwicklerHinzu("ent", "ent2");
    this.anmelden("ent", "ent2");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.passwortAendern("ent2", "ente");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.22"))
            , "Entwickler darf eigenes Passwort aendern");
  }

  @Test
  public void testEntwicklerDarfFalschesPasswortNichtZumAendernNutzen()
      throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.entwicklerHinzu("ent", "ent2");
    this.anmelden("ent", "ent2");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.passwortAendern("enton", "ente");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.23"))
            , "Entwickler darf falsches Passwort nicht zur "
                + "Passwortaenderung nutzen");
  }

  @Test
  public void testEntwicklerKannNeuesPasswortNutzen() throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.entwicklerHinzu("ent", "ent2");
    this.anmelden("ent", "ent2");
    this.passwortAendern("ent2", "ente");
    this.anmelden("admin", "admin");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.anmelden("ent", "ente");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.26"))
            , "Entwickler kann neues Passwort nutzen");
  }

  @Test
  public void testEntwicklerDarfFalschesPasswortNichtNutzen() throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.entwicklerHinzu("ent", "ent2");
    this.anmelden("ent", "ent2");
    this.passwortAendern("ent2", "ente");
    this.anmelden("admin", "admin");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.anmelden("ent", "ent2");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.27"))
            , "Entwickler darf falsches Passwort nicht nutzen");
  }

  @Test
  public void testSysadminKannKorrektesPasswortAendern() throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.sysadminHinzu("ent", "ent2");
    this.anmelden("ent", "ent2");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.passwortAendern("ent2", "ente");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.22"))
            , "Sysadmin darf eigenes Passwort aendern");
  }

  @Test
  public void testSysadminKannFalschesPasswortNichtZumAendernNutzen() 
      throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.sysadminHinzu("ent", "ent2");
    this.anmelden("ent", "ent2");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.passwortAendern("enton", "ente");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.23"))
            , "Sysadmin darf falsches Passwort nicht zur "
                + "Passwortaenderung nutzen");
  }

  @Test
  public void testSysadminKannNeuesPasswortNutzen() throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.sysadminHinzu("ent", "ent2");
    this.anmelden("ent", "ent2");
    this.passwortAendern("ent2", "ente");
    this.anmelden("admin", "admin");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.anmelden("ent", "ente");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.26"))
            , "Sysadmin kann neues Passwort nutzen");
  }

  @Test
  public void testSysadminDarfFalschesPasswortNichtNutzen() throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.sysadminHinzu("ent", "ent2");
    this.anmelden("ent", "ent2");
    this.passwortAendern("ent2", "ente");
    this.anmelden("admin", "admin");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.anmelden("ent", "ent2");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.27"))
            , "Sysadmin darf falsches Passwort nicht nutzen");
  }

  @Test
  public void testProjektadminKannKorrektesPasswortAendern() throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.projektadminHinzu("ent", "ent2");
    this.anmelden("ent", "ent2");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.passwortAendern("ent2", "ente");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.22"))
            , "Projektadmin darf eigenes Passwort aendern");
  }

  @Test
  public void testProjektadminKannFalschesPasswortNichtZumAendernNutzen() 
      throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.projektadminHinzu("ent", "ent2");
    this.anmelden("ent", "ent2");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.passwortAendern("enton", "ente");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.23"))
            , "Projektadmin darf falsches Passwort nicht zur "
                + "Passwortaenderung nutzen");
  }

  @Test
  public void testProjektadminKannNeuesPasswortNutzen() throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.projektadminHinzu("ent", "ent2");
    this.anmelden("ent", "ent2");
    this.passwortAendern("ent2", "ente");
    this.anmelden("admin", "admin");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.anmelden("ent", "ente");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.26"))
            , "Projektadmin kann neues Passwort nutzen");
  }

  @Test
  public void testProjektadminDarfFalschesPasswortNichtNutzen() throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.projektadminHinzu("ent", "ent2");
    this.anmelden("ent", "ent2");
    this.passwortAendern("ent2", "ente");
    this.anmelden("admin", "admin");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.anmelden("ent", "ent2");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.27"))
            , "Projektadmin darf falsches Passwort nicht nutzen");
  }

  @Test
  public void testEntwicklerKannLoginNichtAendern() throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.entwicklerHinzu("ent", "ent2");
    this.anmelden("ent", "ent2");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.loginAendern("ent", "ente");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.11"))
            , "Entwickler darf Login nicht aendern");
  }

  @Test
  public void testSysadminKannKorrektesLoginAendern1() throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.sysadminHinzu("ent", "ent2");
    this.anmelden("ent", "ent2");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.loginAendern("admin", "ente");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.10"))
            , "Sysadmin darf Logins aendern");
  }

  @Test
  public void testSysadminKannKorrektesLoginAendern2() throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.sysadminHinzu("ent", "ent2");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.loginAendern("ent", "ente");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.10"))
            , "Sysadmin darf Logins aendern");
  }

  @Test
  public void testSysadminKannNichtExistentesLoginNichtAendern() throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.sysadminHinzu("ent", "ent2");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.loginAendern("administer", "ente");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.11"))
            , "Sysadmin darf nicht existentes Login nicht aendern");
  }

  @Test
  public void testNutzerKannNeuesLoginNutzen() throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.sysadminHinzu("ent", "ent2");
    this.anmelden("ent", "ent2");
    this.loginAendern("admin", "ente");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.anmelden("ente", "admin");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.26"))
            , "Nutzer muss veraendertes Login nutzen koennen");
  }

  @Test
  public void testNutzerDarfFalschesLoginNichtNutzen() throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.sysadminHinzu("ent", "ent2");
    this.anmelden("ent", "ent2");
    this.loginAendern("admin", "ente");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.anmelden("admin", "admin");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.27"))
            , "Nutzer darf veraendertes/altes Login nicht nutzen");
  }

  @Test
  public void testProjektadminKannLoginNichtAendern() throws Exception {
    this.dialog = new Zugriffsdialog();
    this.anmelden("admin", "admin");
    this.projektadminHinzu("ent", "ent2");
    this.anmelden("ent", "ent2");
    String systemOut = SystemLambda.tapSystemOutNormalized(() -> {
      this.loginAendern("ent", "ente");
    });
    Assertions.assertTrue(systemOut
            .contains(Messages.getString("Zugriffsdialog.11"))
            , "Projektadmin darf Logins nicht aendern");
  }
}
